
/* Portions Copyright (c) 1995,1996,1997 by Trusted Information Systems, Inc.
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND TRUSTED INFORMATION SYSTEMS DISCLAIMS
 * ALL WARRANTIES WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL TRUSTED INFORMATION 
 * SYSTEMS BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL
 * DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR
 * PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS
 * ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
 * SOFTWARE.
 *
 *
 * Trusted Information Systems, Inc. has received approval from the
 * United States Government for export and reexport of TIS/DNSSEC
 * software from the United States of America under the provisions of
 * the Export Administration Regulations (EAR) General Software Note
 * (GSN) license exception for mass market software.  Under the
 * provisions of this license, this software may be exported or
 * reexported to all destinations except for the embargoed countries of
 * Cuba, Iran, Iraq, Libya, North Korea, Sudan and Syria.  Any export
 * or reexport of TIS/DNSSEC software to the embargoed countries
 * requires additional, specific licensing approval from the United
 * States Government. 
 */




static char rcsid[] = "$Header: /home/ogud/sec_bind/dnssec_lib/RCS/rsaref_link.c,v 1.20 1997/01/22 21:32:39 ogud Exp $";
/* 
 * This file contains two components 
 * 1. Interface to the rsaref library to allow compilation of Bind 
 *    with TIS/DNSSEC when RSAREF is not available 
 *    all calls to RSAREF are contained inside this file.
 * 2. The glue to connvert RSA{REF} KEYS to and from external formats
 */
#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <memory.h>
#include <sys/param.h>
#include <netinet/in.h>
#include <sys/time.h>
#ifdef USE_OPTIONS_H
#include "../conf/options.h"
#endif
#include <arpa/nameser.h>
#include <resolv.h> 
#ifdef DNS_SECURITY
#include "ns_defs.h"
#include "db_defs.h"
#endif
#include "dnssec.h"
#include "dns_crypto.h"

#ifdef RSAREF
static void   InitRandomStruct  __P((R_RANDOM_STRUCT *));

/* 
 * rsa_sign() Function to call the appropriate RSAREF signing functions
 */
int 
rsa_sign( mode, context, data , len, priv_key, signature)
int mode, len;
R_SIGNATURE_CTX * context;
R_RSA_PRIVATE_KEY *priv_key;
u_char *data, *signature;
{
  int sign_len = 0;
  if( (mode & SIG_MODE_INIT) && R_SignInit( context, DA_MD5))
      return SIGN_INIT_FAILURE;
      
/* equivalent of SIG_MODE_UPDATE */ 
  if( (data && len > 0 ) &&  R_SignUpdate( context, data, len))
      return SIGN_UPDATE_FAILURE;
      
  if( (mode & SIG_MODE_FINAL) && R_SignFinal( context, signature, 
					     &sign_len, priv_key))
      return SIGN_FINAL_FAILURE;
  return sign_len;
}

/* 
 * rsa_verify() Function to call the appropriate RSAREF verification functions
 */
rsa_verify( mode, context, data, len, pub_key, signature, sig_len)
int mode, len, sig_len;
R_SIGNATURE_CTX * context;
R_RSA_PUBLIC_KEY *pub_key;
u_char *data, *signature;
{
      if( (mode & SIG_MODE_INIT) && R_VerifyInit( context, DA_MD5))
	  return VERIFY_INIT_FAILURE;
      
      if( (data && len > 0 ) && R_VerifyUpdate( context, data, len))
	  return VERIFY_UPDATE_FAILURE;
      
      if( (mode & SIG_MODE_FINAL) && R_VerifyFinal( context, signature, 
						   sig_len, pub_key))
	  return VERIFY_FINAL_FAILURE;
      return 0;
}
/* 
 * rsa2dnskey(): Function to convert key in RSAREF to DNS distribution format
 *         this function gets in a pointer to the public key and an work area
 *         to write the key into
 */
int
rsa2dnskey(public, out_str, out_len )
R_RSA_PUBLIC_KEY *public;
char *out_str;
int out_len;
{
  int n, loc;
  u_char *op, out[RAW_KEY_SIZE];
  op=out;
  if( public == NULL || out_len <=0 )
    return 0;
  for( n=0; public->exponent[n] == '\0'; n++);  /* find first non zero */
  *op++ = (u_char) (MAX_RSA_MODULUS_LEN-n);     /* set lenght of exponent */
  for( ; n < MAX_RSA_MODULUS_LEN; op++)         /* copy exponent */
    *op = public->exponent[n++];
  loc = (public->bits+7)/8;                      /* find beg. of modulus */
  memcpy(op, &public->modulus[MAX_RSA_MODULUS_LEN-loc], loc);/*copy exponent*/
  if( loc*6/8+1+out[0] < out_len)
     n = encode64(out, out_str, ((int)out[0])+1+loc);  /* encode key */
  else
    return 0;
  return n;
}

/* 
 * function that converts from a KEY RR format to a RSA KEY 
 * returns 1 if everything is fine 0 otherwise
 */ 
int
dns2rsakey( len, key, name, s_key)
int len;
u_char *key, *name;
KEY *s_key;
{
  int bytes;
  u_char *key_ptr;
  RSA_Key *r_key;
  if( key == NULL) 
    return 0;
/* allocate the RSA key  if there is no memory */
  if((r_key = (RSA_Key *) malloc( sizeof(RSA_Key))) == NULL){
    perror("dns2rsakey(): Memory allocation error");
    exit(42);
  }    
  memset(r_key, 0, sizeof(RSA_Key));
  s_key->KEY_struct = (void *) r_key; 
  if((s_key->key_name = r_key->signer = (char *) strdup( name)) == NULL) {
    perror("dns2rsakey(): Memory allocation error");
    exit(42);
  }
  r_key->Public_Key = (R_RSA_PUBLIC_KEY *) malloc(sizeof(R_RSA_PUBLIC_KEY));
  if( r_key->Public_Key == NULL) {
    perror("dns2rsakey(): Memory allocation error");
    exit(42);
  }    
  memset(r_key->Public_Key, 0, sizeof (R_RSA_PUBLIC_KEY));
  key_ptr = key;
  bytes = (int) *key_ptr++; /* length of exponent in bytes */
  memcpy(&r_key->Public_Key->exponent[MAX_RSA_MODULUS_LEN-bytes],
	 key_ptr, bytes);

  key_ptr += bytes;         /* beginning of modulus */
  bytes = len - bytes -1;   /* length of modulus */
  memcpy( &r_key->Public_Key->modulus[MAX_RSA_MODULUS_LEN-bytes],key_ptr, 
	 bytes);
  r_key->Public_Key->bits = bytes * 8;
  s_key->foot = (u_int16_t) _getshort( (u_char *) 
		  &r_key->Public_Key->modulus[MAX_RSA_MODULUS_LEN-3]);
  /* 
   * Here I should insert code to find first set bit and 
   *  decrement bits accordingly 
   */
#ifdef NOT_TESTED
  j = (u_int32_t) r_key->Public_Key->modulus[MAX_RSA_MODULUS_LEN-bytes];
  while(j) {
    j>>=1;
    r_key->Public_Key->bits++;
  }
  r_key->Public_Key-=8;
#endif
  s_key->key_size = r_key->Public_Key->bits;
  return 1;
}

/* function to free all dynamicly allocated structures in RSA_Key */
void
Free_RSAKey( r_key)
RSA_Key *r_key;
{
  if( r_key != NULL) {
    if(r_key->Private_Key)     SAFE_FREE(r_key->Private_Key);
    else if(r_key->Public_Key) SAFE_FREE(r_key->Public_Key);
    SAFE_FREE(r_key);
  }
  return ;
}

/* function to encode RSA Key into the portable file format */
int
Encode_RSAKey( rkey, buff, buff_len)
R_RSA_PRIVATE_KEY *rkey;
char *buff;
int buff_len;
{
  char *cp;
  int len;
  memset(buff, 0, buff_len);
  sprintf(buff,"Private-key-format: v%s\nAlgorithm: 1 (RSA)\n",
	  KEY_FILE_FORMAT);
  cp = (char *)strchr(buff,'\0');
  if((len=write_str(cp, "Modulus: ",rkey->modulus, 
		    MAX_RSA_MODULUS_LEN))<=0)    return -1;
  cp+=len;
  if ((len=write_str(cp, "PublicExponent: ",rkey->publicExponent, 
		     MAX_RSA_MODULUS_LEN))<=0)    return -2;
  cp += len;
  if ((len=write_str(cp, "PrivateExponent: ", rkey->exponent, 
		     MAX_RSA_MODULUS_LEN))<=0)   return -3;
  cp += len;
  if ((len=write_str(cp, "Prime1: ", rkey->prime[0], MAX_RSA_PRIME_LEN))<0) 
	 return -4;
  cp += len;
  if ((len=write_str(cp, "Prime2: ",rkey->prime[1], MAX_RSA_PRIME_LEN))<0)
    return -5;
  cp += len;
  if ((len=write_str(cp, "Exponent1: ",rkey->primeExponent[0], 
		     MAX_RSA_PRIME_LEN))<0) return -6;
  cp += len;
  if ((len=write_str(cp, "Exponent2: ",rkey->primeExponent[1], 
		     MAX_RSA_PRIME_LEN))<0) return -7;
  cp += len;
  if ((len=write_str(cp, "Coefficient: ",rkey->coefficient, 
           MAX_RSA_PRIME_LEN))<0) return -8;
  cp += len;
  return (int) (cp-buff);
}

int
Decode_RSAKey( r_key, buff, buff_len)
RSA_Key *r_key;
char *buff;
int buff_len;
{
  char *p = buff;
  R_RSA_PRIVATE_KEY key;

  int foot = -1;
  memset( &key, 0, sizeof(key));
  if (!verify_str(&p, "Modulus: ")) return -1;
  if (!read_str(&p, key.modulus, MAX_RSA_MODULUS_LEN)) return -2;
  key.bits = calculate_bits(key.modulus, MAX_RSA_MODULUS_BITS);
  while( *p && p < &buff[buff_len]) {
    if (verify_str(&p, "PublicExponent: ")) {
      if (!read_str(&p, key.publicExponent, MAX_RSA_MODULUS_LEN)) 
	return -3;
    }
    else if (verify_str(&p, "PrivateExponent: ")) {
      if (!read_str(&p, key.exponent, MAX_RSA_MODULUS_LEN)) 
	return -4;
    }
    else if (verify_str(&p, "Prime1: ")) {
      if (!read_str(&p, key.prime[0], MAX_RSA_PRIME_LEN)) 
	return -5;
    }
    else if (verify_str(&p, "Prime2: ")) {
      if (!read_str(&p, key.prime[1], MAX_RSA_PRIME_LEN)) 
	return -6;
    }
    else if (verify_str(&p, "Exponent1: ")){
      if (!read_str(&p, key.primeExponent[0], MAX_RSA_PRIME_LEN))
	return -7;
    }
    else if (verify_str(&p, "Exponent2: ")) {
      if (!read_str(&p, key.primeExponent[1], MAX_RSA_PRIME_LEN)) 
	return -8;
    }
    else if (verify_str(&p, "Coefficient: ")) {
      if (!read_str(&p, key.coefficient, MAX_RSA_PRIME_LEN)) 
	return -9;
    }
    else {
      printf("Decode_RSAKey(): Bad keyword %s\n", p);
      return -10;
    }
  }  /* while (p  */
  r_key->Private_Key = 
    (R_RSA_PRIVATE_KEY *) malloc( sizeof(R_RSA_PRIVATE_KEY));
  if(r_key->Private_Key == NULL) {
    perror("dns2rsakey(): Memory allocation error");
    exit(42);
  }    
  memset( r_key->Private_Key, 0, sizeof(*r_key->Private_Key ));
  r_key->Public_Key = (R_RSA_PUBLIC_KEY *)  r_key->Private_Key ;
  memcpy( r_key->Private_Key, &key, sizeof(R_RSA_PRIVATE_KEY));
  foot = (int) (u_short) _getshort( 
	     &r_key->Public_Key->modulus[MAX_RSA_MODULUS_LEN-3]);
  return foot;
}

/*
 * Function used to generate new keys
 *
 * Generate_RSA_KEY() does all the necesary stuff to generate unique keys that
 * are hard to predict 
 * Parameters
 *   key  : generic Key structure
 *   exp  : the public exponent
 * Return 
 *     0 Failure 
 *     1 Success
 */
int 
Generate_RSAKey(key, exp)
    KEY *key;
     int exp;
{  
  R_RSA_PUBLIC_KEY *public;
  R_RSA_PRIVATE_KEY *private;
  R_RSA_PROTO_KEY proto;
  R_RANDOM_STRUCT randomStruct;
  RSA_Key *rsa;
  int status;

  if((public = (R_RSA_PUBLIC_KEY *) malloc(sizeof(R_RSA_PUBLIC_KEY))) == NULL){
    perror("Generate_RSAKey(): Memory allocation error");
    exit(42);
  }    
  if((private =(R_RSA_PRIVATE_KEY *) malloc(sizeof(R_RSA_PRIVATE_KEY)))==NULL){
    perror("Generate_RSAKey(): Memory allocation error");
    exit(42);
  }    
  memset( public, 0, sizeof( *public ));
  memset( private, 0, sizeof( *private ));
  proto.bits = key->key_size;
  proto.useFermat4 = exp ?  0x1 : 0x0;  /* 1 for f4=65537, 0 for f0=3 */
  printf("\nGen_RSA_KEY: Generating KEY for %s Please wait\n", key->key_name);
  InitRandomStruct(&randomStruct);
  status = R_GeneratePEMKeys(public, private, &proto, &randomStruct);
  if ( status) {
    printf("Gen_RSA_KEY:No Key Pair generated %d\n",status);
    SAFE_FREE(public);
    SAFE_FREE(private);
    return 0;
  }
  if((rsa = (RSA_Key *) malloc(sizeof(RSA_Key)))==NULL){
    perror("Generate_RSAKey(): Memory allocation error");
    exit(42);
  }     
  memset( rsa, 0, sizeof(*rsa ));
  rsa->signer      = key->key_name;
  rsa->Private_Key = private;
  rsa->Public_Key  = public;
  key->KEY_struct  = (void *) rsa;
  return 1;
}

int 
rsa_cmp( rkey1, rkey2) 
RSA_Key *rkey1, *rkey2;
{
  return memcmp(rkey1->Public_Key, rkey2->Public_Key, 
		sizeof(R_RSA_PUBLIC_KEY));
}
/*
 * This routine gets a bunch of system values to randomize the randomstruct
 * I use number of system calls to get somewhat unpredicable values
 * Then I call a special function prandom() that will do some magic depending
 * on the system used. 
 * If this function is executed on reasonably busy machine then the values
 * that prandom uses are hard to 
 *     1. Predict 
 *     2. Regenerate
 *     3. Hard to spy on as nothing is stored to disk and data is consumed
 *        as fast as it is generated.
 */
static void 
InitRandomStruct (randomstruct)
R_RANDOM_STRUCT *randomstruct;
{
  unsigned bytesNeeded;
  struct timeval tv;
  R_RandomInit (randomstruct);

  /* The runtime of the script is unpredictable within some range 
   * thus I'm getting the time of day again as this is an hard to guess 
   * value and the number of characters of the output from the script is
   * hard to guess. 
   * This must be the FIRST CALL 
   */
  gettimeofday(&tv, 0);
  R_RandomUpdate (randomstruct, (u_char *) &tv, sizeof(struct timeval));
  /* 
   * starting the script and performing stat() calls first adds 
   * unpredicatbility to the time gettimeofday is called
   * then we get some system parameters that an attacker may or may not know
   * it he/she has access to the system 
   */
  prandom( randomstruct, R_RandomUpdate);
  /* The runtime of the script is unpredictable within some range 
   * thus I'm getting the time of day again as this is an hard to guess 
   * value and the number of characters of the output from the script is
   * hard to guess. 
   * This must be the LAST CALL
   */
  gettimeofday(&tv, 0);
  R_RandomUpdate (randomstruct, (u_char *) &tv, sizeof(struct timeval));

  R_GetRandomBytesNeeded(&bytesNeeded, randomstruct);
  if (bytesNeeded) {
    printf("InitRandomStruct() didn't initialize enough randomness\n");
    exit(33);
  }
}

/*
 * key is the key to use 
 * [io]buf are the input and output buffers
 * flags selects weither  to use des program mode or hash mode 
 * mode == 0 is decrypt(DES_DECRYPT)  ==1 is encrypt (DES_ENCRYPT)
 */
int 
des_function( key, ibuf, obuf, len, mode)
u_char *key, *ibuf, *obuf;
int len, mode;
{
  DES_CBC_CTX context;  
  u_char zero[8];
  int i,v;
  v = i = 8-(len&0x7);
  memset(zero,0,sizeof(zero));

  DES_CBCInit( &context, key, zero, mode);
  v = 8-v;
  if( mode ==DES_ENCRYPT) { /* mode 0 == decrypt  1 == encrypt */
    len += i;
    ibuf[len-1] = v; /* counts the number of bytes from this block to keep*/
  }
  DES_CBCUpdate( &context, obuf, ibuf, len);
  memset(&context,0, sizeof(context));
  return len;
}
#else  /* RSAREF */
int 
des_function ( key, ibuf, obuf, len, mode) 
u_char *key, *ibuf, *obuf;
int len, mode ;
{
  memcpy( obuf, ibuf, len);
  return len;
}
#endif /* RSAREF */
