
/* Portions Copyright (c) 1995,1996,1997 by Trusted Information Systems, Inc.
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND TRUSTED INFORMATION SYSTEMS DISCLAIMS
 * ALL WARRANTIES WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL TRUSTED INFORMATION 
 * SYSTEMS BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL
 * DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR
 * PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS
 * ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
 * SOFTWARE.
 *
 *
 * Trusted Information Systems, Inc. has received approval from the
 * United States Government for export and reexport of TIS/DNSSEC
 * software from the United States of America under the provisions of
 * the Export Administration Regulations (EAR) General Software Note
 * (GSN) license exception for mass market software.  Under the
 * provisions of this license, this software may be exported or
 * reexported to all destinations except for the embargoed countries of
 * Cuba, Iran, Iraq, Libya, North Korea, Sudan and Syria.  Any export
 * or reexport of TIS/DNSSEC software to the embargoed countries
 * requires additional, specific licensing approval from the United
 * States Government. 
 */




/* Key_Gen.c - based on Rdemo.c RSAREF demonstration program
 * $Header: /home/ogud/sec_bind/signer/RCS/key_gen.c,v 1.25 1997/01/22 22:04:47 ogud Exp $
 */
#include <stdio.h>
#include <memory.h>
#include <string.h>
#include <errno.h>
#include <fcntl.h>
#include <netinet/in.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/time.h>
#include "arpa/nameser.h"
#include "dnssec.h"

#define KEY_SIZE  512  /* 512 768  1024 */
int do_axfr=0;
int gen_key=0;
void usage();
int ReadPublicKey            __P((char *, u_char *, int *, int *, int *));
char *prog, key[KEY_SIZE];

int 
main (argc, argv)
int argc;
char *argv[];
{
  int bytes,i, len; /* no default values */
  char *name=NULL, enc_key[KEY_SIZE*2], *fmt=
    "%s.\t IN KEY %d (;FLAGS\n\t\t%d\t;Protocol\n\t\t%d\t;Alg\n\t%s\n\t)\n\n";
  FILE *fd; 
  int zone_key=0, user_key = 0, end_key = 0; 
  int experimental=0; /* either this key is on or key is mandedtory */ 
  int signer=0, size=KEY_SIZE, exp=0; 
  int flags=0, protocol=255, sign_val = 0, alg=1; /* default values */ 
  char file[256];
  struct stat *buf;

  memset(key, 0, sizeof(key)); 
  memset(enc_key, 0, sizeof(enc_key));
  /* process arguments */ 
  if( (prog= strrchr(argv[0],'/')) ==   NULL) prog=argv[0];
  else prog++;
  i=1;
  while( i < argc) {
    if(argv[i][0] == '-') {
      switch(tolower(argv[i][1])){ 
      case 'a': 
	flags |= SIGN_F_NOAUTHENT;
	break;
      case 'c': 
/*	flags |= SIGN_F_NOCONFIDE; */
	exp = 1;
	break;
      case 'e': end_key = 1;
	flags |= SIGN_F_END;
	break;
      case 'f': 
	if( !gen_key) 
	  printf("This flag -f is ignored when not specified with the -g flag\n");
	exp=1;
	break;
      case 'g':
	gen_key++;
	if( argc > i && isdigit(argv[i+1][0])) size = atoi(argv[++i]);
	else size = KEY_SIZE;
	break;
      case 'i': 
	flags |= SIGN_F_IPSEC;
	break;
      case 'm': 
	flags |= SIGN_F_MOSS;
	break;
      case 'p': 
	if(isdigit(argv[i+1][0])) protocol = atoi(argv[++i]);
	else { 
	  printf("%s: -p flag not followed by a number %s\n", prog,argv[i+1]);
	  usage();
	} 
	break;
      case 's':
	signer = 1;
	/* Default: not singatory key */
	if(isdigit(argv[i+1][0])) sign_val = atoi(argv[++i]);
	else {
	  printf("%s: -s flag not followed by a number %s Default:1 used\n", 
		 prog, argv[i+1]);
	  sign_val = 1;
	} 
	if( sign_val < 1 || sign_val > 15) { 
	  printf("%s: Signatory value %d out of range1..15]\n", prog, sign_val);
	  usage();
	} 
	if( sign_val & 0xe) /* if one of bits 321 is set bit 0 can not be set*/
	  sign_val &= 0xe; 
	flags |= sign_val;
	break;
      case 'u' : 
	user_key = 1;
	flags |= SIGN_F_USER;
	break ;
      case 'x': 
	experimental = 1;
	flags |= SIGN_F_EXPER;
	break;
      case 'z': 
	zone_key = 1;
	flags |= SIGN_F_ZONE;
	break;
      default:
	printf("Unknown switch %s\n",argv[i]);
	usage();
      } /* switch */
    }
    else 
      name = strdup(argv[i]);
    i++;
  }	/* while( argc */ 
  if( name==NULL) {
    printf("No key name specified\n");
    usage();
  }

  while( name[0] == '\\') {
    char *tmp = name;
    name = strdup(&name[1]);
    free(tmp);
  }
  len = strlen(name);
  while( len && name[--len] == '.') 
    name[len] = '\0';
  /* now validate the flags */ 
  if( gen_key) { 
    if( zone_key + user_key + end_key !=1) { /* Key type wrong */ 
      if( zone_key + user_key + end_key ==0) 
	printf("%s: You need to specify one of -zue\n", prog);
      else printf("%s: You can only specify one of -zue\n",prog);
      usage();
    } 
  }

  if( protocol < 0 || protocol > 255) { 
    printf("%s: Protocol value %d out of range [0..255]\n", prog, protocol);
    usage();
  } 
  if( zone_key && !signer) flags |= 0x01;
/* I need to make this compatable with SECPDATE*/
  if( flags & SIGN_F_NOCONFIDE) exp = 0; 
  else if( flags & SIGN_F_NOAUTHENT && exp==0) flags |=  SIGN_F_NOCONFIDE;
  if( signer && (user_key | end_key)) 
    printf( "Warning: User/End  key is allowed to sign\n");
  if( (flags & (SIGN_F_NOAUTHENT|SIGN_F_NOCONFIDE)) == 
               (SIGN_F_NOAUTHENT|SIGN_F_NOCONFIDE)){ 
    printf("Null key requested Do SOMETHING\n");
  } 
  else if(gen_key) {
    sprintf( file, "%s.priv",name);
    if(stat(file, buf) == 0) { 
      printf("key_gen: Not allowed to overwrite existing KEY %s\n",file);
      exit (4);
    } /* create a public/private key pair */ 
    printf("Generating %d bit Key for %s\n\n",size, name);
    if(!dnssec_genkey(name, size, exp, flags, protocol, 1)) { 
      printf("Failed generatingkey for %s\n", name);
      exit(12);
    } 
  } 
  bytes = ReadPublicKey(name, enc_key, &flags, &protocol, &alg);
  if(bytes >= 0) { 
    printf(fmt, name, flags, protocol, alg, enc_key);
    if( gen_key){ 
      fd = fopen("Keys","a+");
      fprintf(fd, fmt, name, flags, protocol, alg, enc_key);
      fclose(fd);
    }
  } 
  exit(0);
}

void 
usage(){
  printf( 
 "Usage:\n%s: [-acimx] [-g [size] -f] {-z -e -u} [-p no] [-s no] name\n",
	 prog);
  printf(
  "\t -g Generate keys if flag is not specified Key will be read from file\n");
  printf("\t size:\tthe size of the key in bits[Default:512]\n");
  printf("\t -a Key is not allowed to be used for authenticaiton\n");
  printf("\t -c Key is not allowed to be used for confidenciality\n");
  printf("\t -z Zone key \n");
  printf("\t -e End key \n");
  printf("\t -u User key \n");
  printf("\t -i Key can be used with IPSEC protocols\n");
  printf("\t -m Key can be used with MOSS\n");
  printf("\t -f For RSA KEYS only:Use Fermat(4) as modulus 65537, [Default:3]\n");
  printf("\t -p Set protocol field to <no>, [Default:255]\n");
  printf(
     "\t -s Strength Value this key is allowed to sign with [Default:1]\n");
  printf("\t -x Experimental Key\n");
  printf("\t name: who the key is for\n");
  exit (-3);
}


int 
ReadPublicKey(name, key, flag, prot, algo)
char *name; 
u_char *key;
int *flag, *prot, *algo;
{
  char file[500];
  FILE *fd;
  int r_flag, r_algo, r_prot; /* the parameters from the input file */ 
  int i, len=0;

  sprintf(file,"%s.public", name); 
  fd = fopen(file,"r");
  if( fd ) {
    if((i = fscanf(fd,"%d %d %d\n",&r_flag, &r_prot, &r_algo)) != 3) 
      printf("ReadPublicKey:Format error expecting 3 numeric input fields got %d\n",i);
    else if( r_algo != *algo) 
      printf("ReadPublicKey:Key of wrong algorithim type %d != %d\n",
	     *algo, r_algo);
    else { 
      *flag |= r_flag; /* Hum not sure this is the right thing to do XXXX */
      if( r_prot == 253) 
	*prot = 253;  /* no key can only stay no key */
      else if( *prot== 255 || *prot ==0)
	*prot = r_prot; /* overwrite if defaults */
      i =  fscanf(fd,"%s",key);
      len = strlen(key)-1; 
      while(len > 0 && key[len] <'+')  /* strip trailing garbage */
	key[len--] = 0x0;
      len++;
    }
    fclose(fd);
    return len;
  }
  printf("key_gen:ReadPublicKey() Could not open file %s\n",file);
  return -1;
}



